# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.facebookaccesstokenanalyzer import FacebookAccessTokenAnalyzer


class TestFacebookAccessTokenAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = FacebookAccessTokenAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_negative(self):
        # random string
        self.paste.body = "Test"
        self.assertFalse(self.analyzer.match(self.paste))

        # empty
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        # non-alphanumeric
        self.paste.body = "EAACEdEose0cBA!abcdef"
        self.assertFalse(self.analyzer.match(self.paste))

        # invalid prefix
        self.paste.body = "EAAACEdEose0cBAabcdef"
        self.assertFalse(self.analyzer.match(self.paste))

        # invalid params
        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste = None
        self.assertFalse(self.analyzer.match(self.paste))

    def test_intext(self):
        """Test if matches inside text are recognized"""
        self.paste.body = "This is my access token: EAACEdEose0cBA0123456789ABCDEFabcdef, please don't lose it!"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("EAACEdEose0cBA0123456789ABCDEFabcdef", match[0])

    def test_multiple(self):
        """Test if multiple matches are recognized"""
        self.paste.body = "This is my access token: EAACEdEose0cBA0123456789ABCDEFabcdef, please don't lose it! " \
                          "Also there is another one: EAACEdEose0cBAfacebooksucks so I guess it's ok"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("EAACEdEose0cBA0123456789ABCDEFabcdef", match[0])
        self.assertEqual("EAACEdEose0cBAfacebooksucks", match[1])

    def test_match_positive(self):
        # random valid string
        self.paste.body = "EAACEdEose0cBA0123456789ABCDEFabcdef"
        self.assertTrue(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from .logicalbaseanalyzer import LogicalBaseAnalyzer


class OrAnalyzer(LogicalBaseAnalyzer):
    """Meta analyzer which matches a paste if any of the passed analyzers match that paste"""
    name = "OrAnalyzer"

    def match(self, paste):
        """Returns True if all of the passed analyzers matched"""
        for analyzer in self.analyzers:
            if analyzer.match(paste):
                return True

        return False



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class UplayKeyAnalyzer(RegexAnalyzer):
    """Analyzer to match uplay keys via regex."""
    name = "UplayKeyAnalyzer"

    def __init__(self, actions):
        regex = r"\b(?<!-)[A-Z0-9]{4}\-[A-Z0-9]{4}\-[A-Z0-9]{4}\-[A-Z0-9]{4}\b(?!-)"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class AzureSubscriptionKeyAnalyzer(RegexAnalyzer):
    """
    Analyzer to match Azure subscription key via regex

    Keys are 32 character alphanumeric (lower case)
    """
    name = "AzureSubscriptionKeyAnalyzer"

    def __init__(self, actions):
        # https://docs.microsoft.com/en-us/azure/api-management/api-management-subscriptions
        regex = r"\b[a-f0-9]{32}\b"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
import unittest

from pastepwn.util.dictwrapper import DictWrapper


class TestDictwrapper(unittest.TestCase):
    def setUp(self):
        self.test_dict = {"key1": "value1", "key2": "value2"}

    def test_existing_key(self):
        test_wrapped = DictWrapper(self.test_dict)
        self.assertEqual(test_wrapped["key1"], "value1")

    def test_non_existing_key(self):
        test_wrapped = DictWrapper(self.test_dict)
        # Make sure that retreiving a valid key works
        self.assertEqual(test_wrapped["key1"], "value1")
        # Check if retreiving a nonexistent value works as expected
        self.assertEqual(test_wrapped["key3"], "${key3}")
        # Make sure that retreiving a valid key still works
        self.assertEqual(test_wrapped["key2"], "value2")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.stripeapikeyanalyzer import StripeApiKeyAnalyzer


class TestStripeApiKeyAnalyzer(unittest.TestCase):

    def setUp(self):
        self.analyzer = StripeApiKeyAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # steam key dump
        self.paste.body = "sk_test_4fsrdffsdf345345dfgfg34i"
        self.assertTrue(self.analyzer.match(self.paste))

        # steam key dump
        self.paste.body = "rk_test_4fsrdffsdf345345dfgfg34i"
        self.assertTrue(self.analyzer.match(self.paste))

        # steam key dump
        self.paste.body = "sk_live_4fsrdffsdf345345dfgfg34i"
        self.assertTrue(self.analyzer.match(self.paste))

        # steam key dump
        self.paste.body = "rk_live_4fsrdffsdf345345dfgfg34i"
        self.assertTrue(self.analyzer.match(self.paste))

        # steam key dump
        self.paste.body = "sk_test_YUTGF76uyh876Tyg87T786Tu"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_intext(self):
        """Test if matches inside text are recognized"""
        self.paste.body = "There is my api key: sk_test_YUTGF76uyh876Tyg87T786Tu - take good care of it"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("sk_test_YUTGF76uyh876Tyg87T786Tu", match[0])

    def test_multiple(self):
        """Test if multiple matches are recognized"""
        self.paste.body = "There is my api key: sk_test_YUTGF76uyh876Tyg87T786Tu - take good care of it. The other one is sk_live_4fsrdffsdf345345dfgfg34i"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual(2, len(match))
        self.assertEqual("sk_test_YUTGF76uyh876Tyg87T786Tu", match[0])
        self.assertEqual("sk_live_4fsrdffsdf345345dfgfg34i", match[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "sk_test_YUTGF76uyh876"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid first letter
        self.paste.body = "pk_test_YUTGF76uyh876Tyg87T786Tu"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid state
        self.paste.body = "sk_staging_YUTGF76uyh876Tyg87T786Tu"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.googleapikeyanalyzer import GoogleApiKeyAnalyzer


class TestGoogleApiKeyAnalyzer(unittest.TestCase):

    def setUp(self):
        self.analyzer = GoogleApiKeyAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # google key dump
        self.paste.body = "AIzaSyCTmst6SvsOAQanZKNt-2pt6nuLoFf2kSA"
        self.assertTrue(self.analyzer.match(self.paste))

        # google key dump
        self.paste.body = "AIzaSyBKNst9JE89f4lHuNXQFTUgZKh8VZpvR6M"
        self.assertTrue(self.analyzer.match(self.paste))

        # google key dump
        self.paste.body = "AIzammmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm"
        self.assertTrue(self.analyzer.match(self.paste))

        # google key dump
        self.paste.body = "AIza00000000000000000000000000000000000"
        self.assertTrue(self.analyzer.match(self.paste))

        # key in the middle of a string
        self.paste.body = "api key: AIza00000000000000000000000000000000000"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_intext(self):
        """Test if matches inside text are recognized"""
        self.paste.body = "A key inside a AIzaSyBKNst9JE89f4lHuNXQFTUgZKh8VZpvR6M medium sized text!"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual(1, len(match))
        self.assertEqual("AIzaSyBKNst9JE89f4lHuNXQFTUgZKh8VZpvR6M", match[0])

    def test_multiple(self):
        """Test if multiple matches are recognized"""
        self.paste.body = "A key inside a AIzaSyBKNst9JE89f4lHuNXQFTUgZKh8VZpvR6M medium sized text! Also how about " \
                          "that one: AIzaSyCTmst6SvsOAQanZKNt-2pt6nuLoFf2kSA"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual(2, len(match))
        self.assertEqual("AIzaSyBKNst9JE89f4lHuNXQFTUgZKh8VZpvR6M", match[0])
        self.assertEqual("AIzaSyCTmst6SvsOAQanZKNt-2pt6nuLoFf2kSA", match[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid start
        self.paste.body = "aiza00000000000000000000000000000000000"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid start
        self.paste.body = "000000000000000000000000000000000000000"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "AIzammmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "AIzammmmmmmmm"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import os
import socket
import unittest
from unittest.mock import Mock

import requests.packages.urllib3.util.connection as urllib3_cn

from pastepwn.util import enforce_ip_version


@unittest.skipIf("TRAVIS" in os.environ and os.environ["TRAVIS"] == "true", "Skipping this test on Travis CI.")
class TestNetwork(unittest.TestCase):
    """Tests for the network utils - these tests might fail on CI"""

    def test_enforce_ip_version_4(self):
        enforce_ip_version(4)
        self.assertEqual(urllib3_cn.allowed_gai_family(), socket.AF_INET)
        enforce_ip_version("4")
        self.assertEqual(urllib3_cn.allowed_gai_family(), socket.AF_INET)

    def test_enforce_ip_version_5(self):
        with self.assertRaises(Exception):
            enforce_ip_version(5)

        with self.assertRaises(Exception):
            enforce_ip_version("5")

    def test_enforce_ip_version_6(self):
        enforce_ip_version(6)
        self.assertEqual(urllib3_cn.allowed_gai_family(), socket.AF_INET6)
        enforce_ip_version("6")
        self.assertEqual(urllib3_cn.allowed_gai_family(), socket.AF_INET6)

    def test_enforce_ip_version_None(self):
        enforce_ip_version(None)
        self.assertEqual(urllib3_cn.allowed_gai_family(), socket.AF_UNSPEC)

        enforce_ip_version()
        self.assertEqual(urllib3_cn.allowed_gai_family(), socket.AF_UNSPEC)

    def test_enforce_ip_version_obj(self):
        mock = Mock()
        with self.assertRaises(ValueError):
            enforce_ip_version(mock)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class SHAHashAnalyzer(RegexAnalyzer):
    """
    Analyzer to match SHA password hashes via regex

    Matches SHA 1, 224, 256, 384, and 512 hashes.
    """
    name = "SHAHashAnalyzer"

    def __init__(self, actions):
        regex = r"\b(?<!-)([a-f0-9]{40}|[a-f0-9]{56}|[a-f0-9]{64}|[a-f0-9]{96}|[a-f0-9]{128})\b(?!-)"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
import unittest
from unittest.mock import Mock

from pastepwn.util.listify import listify


class ListifyTest(unittest.TestCase):

    def test_None(self):
        self.assertEqual([], listify(None), "Listify did not return empty list!")

    def test_list(self):
        obj = Mock()
        obj2 = Mock()
        obj3 = Mock()
        obj_list = [obj, obj2, obj3]

        self.assertEqual(obj_list, listify(obj_list), "Listify did not return the given list!")

    def test_single(self):
        obj = Mock()
        self.assertEqual([obj], listify(obj), "Listify did not return single object as list!")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from pastepwn.util import listify
from .basicanalyzer import BasicAnalyzer


class WordAnalyzer(BasicAnalyzer):
    """Analyzer to match the content of a paste by words"""
    name = "WordAnalyzer"

    def __init__(self, actions, words, blacklist=None, case_sensitive=False):
        super().__init__(actions, "{0} ({1})".format(self.name, words))
        self.words = listify(words)

        self.blacklist = blacklist or []
        self.case_sensitive = case_sensitive

    def _blacklist_word_found(self, text):
        if not self.case_sensitive:
            text = text.lower()
            self.blacklist = [x.lower() for x in self.blacklist]

        for word in self.blacklist:
            if word in text:
                return True

        return False

    def add_word(self, word):
        """Add a word to the analyzer.
        :param word: Word to be added
        :return:
        """
        self.words.append(word)

    def match(self, paste):
        """Check if the specified words are part of the paste text"""
        if paste is None:
            return False

        paste_content = paste.body or ""

        if self._blacklist_word_found(paste_content):
            return False

        _matches = []
        if self.case_sensitive:
            for word in self.words:
                # Never use 'return word in paste_content' - otherwise you will
                # return false before all words have been checked
                if word in paste_content:
                    _matches.append(word)
        else:
            for word in self.words:
                if word.lower() in paste_content.lower():
                    _matches.append(word)

        return _matches



# -*- coding: utf-8 -*-
import pathlib

from pastepwn.util import TemplatingEngine
from .basicaction import BasicAction


class SaveFileAction(BasicAction):
    """Action to save each paste as a file named '<pasteID>.txt'"""
    name = "SaveFileAction"

    def __init__(self, path, file_ending=".txt", template=None):
        """
        Action to save each paste as a file named '<pasteID>.txt'
        If you want to store metadata within the file, use template strings
        > https://github.com/d-Rickyy-b/pastepwn/wiki/Templating-in-actions
        :param path: The directory in which the file(s) should be stored
        :param template: A template string describing how the paste variables should be filled in
        """
        super().__init__()
        self.path = pathlib.Path(path)
        self.file_ending = file_ending
        self.template = template or "${body}"

    @staticmethod
    def _remove_prefix(input_string, prefix):
        """Remove a prefix from a certain string (e.g. remove '.' as prefix from '.txt')"""
        if input_string.startswith(prefix):
            return input_string[len(prefix):]
        return input_string

    def get_file_content(self, paste, analyzer_name, matches):
        """Returns the content to be written to the file"""
        return TemplatingEngine.fill_template(paste, analyzer_name, template_string=self.template, matches=matches)

    def perform(self, paste, analyzer_name=None, matches=None):
        """
        Stores the paste as a file
        :param paste: The paste passed by the ActionHandler
        :param analyzer_name: The name of the analyzer which matched the paste
        :param matches: List of matches returned by the analyzer
        :return: None
        """
        if not self.path.exists():
            self.path.mkdir(parents=True, exist_ok=True)

        self.file_ending = self._remove_prefix(self.file_ending, ".")

        if self.file_ending == "":
            file_name = str(paste.key)
        else:
            file_name = "{0}.{1}".format(paste.key, self.file_ending)

        file_path = self.path / file_name
        content = self.get_file_content(paste, analyzer_name, matches)
        with open(file_path, "w", encoding="utf-8") as file:
            file.write(content)



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.uplaykeyanalyzer import UplayKeyAnalyzer


class TestUplayKeyAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = UplayKeyAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # working uplay key
        self.paste.body = "MB9C-LV3C-4RG8-FME8"
        self.assertTrue(self.analyzer.match(self.paste))

        # part of a sentence
        self.paste.body = "Hey, I have your key right here: MB9C-LV3C-4RG8-FME8!"
        self.assertTrue(self.analyzer.match(self.paste))

        # Newline seperated uplay key
        self.paste.body = "MB9C-LV3C-4RG8-FME8\nMB9C-LV3C-4RG8-FME8"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("MB9C-LV3C-4RG8-FME8", match[0])
        self.assertEqual("MB9C-LV3C-4RG8-FME8", match[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "MB9AC-LV3C-4RG8-FME8"
        self.assertFalse(self.analyzer.match(self.paste))

        # Lower-case
        self.paste.body = "mb9c-lv3c-4rg8-fme8"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.base64asciianalyzer import Base64AsciiAnalyzer


class TestBase64AsciiAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = Base64AsciiAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # base64 encoded string: "Hello World" (UTF-8, LF)
        self.paste.body = "SGVsbG8gV29ybGQ="
        self.assertTrue(self.analyzer.match(self.paste))

        # base64 encoded string: "Hello\nWorld" (UTF-8, LF)
        self.paste.body = "SGVsbG9cbldvcmxk"
        self.assertTrue(self.analyzer.match(self.paste))

        # base64 encoded string (32 chars): "2fwZ_CTjDKxu48FLCLZcGdB!sEj5XRQh" (UTF-8, LF)
        self.paste.body = "MmZ3Wl9DVGpES3h1NDhGTENMWmNHZEIhc0VqNVhSUWg="
        self.assertTrue(self.analyzer.match(self.paste))

        # base64 encoded string (64 chars): "Mv=ZH?NJrrBSdhus*KVg%4dG6*C&ub?sSeq!VrzCb_-QcY^KWfxKy8AJ3=^5?b6N" (UTF-8, LF)
        self.paste.body = "TXY9Wkg/TkpyckJTZGh1cypLVmclNGRHNipDJnViP3NTZXEhVnJ6Q2JfLVFjWV5LV2Z4S3k4QUozPV41P2I2Tg=="
        self.assertTrue(self.analyzer.match(self.paste))

        # base64 encoded string (256 chars): "etFk!?m@A_vvdMT39Mgcynx_AFz6HY!4R8U3n_7JA?-rF=F3ehWat%4rKfhsuCc98G
        # =t8jMY7hgJDZ2c!y!$!XQATbk6fQD2pa+EdQ_rfP^&_DKJ34dFPcuGjDBTqdxZ&=3U%@dm&?JW#+k@mB%a3TFn%GAzukL+-%TUTq?fAbAKr
        # @y%LPK+KEmxeh+rg7?s3aR2v5A%tbn&_7zNMckCPRd&s8$wW5Bec@aRMCs@4rn?cRx?a&y-Z%kn&h8aLu*R" (UTF-8, LF)
        self.paste.body = "ZXRGayE/bUBBX3Z2ZE1UMzlNZ2N5bnhfQUZ6NkhZITRSOFUzbl83SkE/LXJGPUYzZWhXYXQlNHJLZmhzdUNjO" \
                          "ThHPXQ4ak1ZN2hnSkRaMmMheSEkIVhRQVRiazZmUUQycGErRWRRX3JmUF4mX0RLSjM0ZEZQY3VHakRCVHFkeF" \
                          "omPTNVJUBkbSY/SlcjK2tAbUIlYTNURm4lR0F6dWtMKy0lVFVUcT9mQWJBS3JAeSVMUEsrS0VteGVoK3JnNz9" \
                          "zM2FSMnY1QSV0Ym4mXzd6Tk1ja0NQUmQmczgkd1c1QmVjQGFSTUNzQDRybj9jUng/YSZ5LVola24maDhhTHUqUg=="
        self.assertTrue(self.analyzer.match(self.paste))

    def test_intext(self):
        """Test if matches inside text are recognized"""
        self.paste.body = "I wan to tel you tha TXY9Wkg/TkpyckJTZGh1cypLVmclNGRHNipDJnViP3NTZXEhVnJ6Q2JfLVFjWV5LV2Z4S3k4QUozPV41P2I2Tg== is " \
                          "very important"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("TXY9Wkg/TkpyckJTZGh1cypLVmclNGRHNipDJnViP3NTZXEhVnJ6Q2JfLVFjWV5LV2Z4S3k4QUozPV41P2I2Tg==", match[0])

    def test_multiple(self):
        """Test if multiple matches are recognized"""
        # Needed to keep the words below 3 chars each. Otherwise they would match as well
        self.paste.body = "I wan to tel you tha TXY9Wkg/TkpyckJTZGh1cypLVmclNGRHNipDJnViP3NTZXEhVnJ6Q2JfLVFjWV5LV2Z4S3k4QUozPV41P2I2Tg== is " \
                          "ver imp.\nBut not onl tha, it's als MmZ3Wl9DVGpES3h1NDhGTENMWmNHZEIhc0VqNVhSUWg= and muc mor!"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("TXY9Wkg/TkpyckJTZGh1cypLVmclNGRHNipDJnViP3NTZXEhVnJ6Q2JfLVFjWV5LV2Z4S3k4QUozPV41P2I2Tg==", match[0])
        self.assertEqual("MmZ3Wl9DVGpES3h1NDhGTENMWmNHZEIhc0VqNVhSUWg=", match[1])

    def test_multiple_min_len(self):
        """Test if we can match multiple base64 strings in a longer text with min_len"""
        analyzer = Base64AsciiAnalyzer(None, min_len=8)
        self.paste.body = "I wanted to tell you that TXY9Wkg/TkpyckJTZGh1cypLVmclNGRHNipDJnViP3NTZXEhVnJ6Q2JfLVFjWV5LV2Z4S3k4QUozPV41P2I2Tg== is " \
                          "very important.\nBut not only that, it's also MmZ3Wl9DVGpES3h1NDhGTENMWmNHZEIhc0VqNVhSUWg= and much more!"
        match = analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("TXY9Wkg/TkpyckJTZGh1cypLVmclNGRHNipDJnViP3NTZXEhVnJ6Q2JfLVFjWV5LV2Z4S3k4QUozPV41P2I2Tg==", match[0])
        self.assertEqual("MmZ3Wl9DVGpES3h1NDhGTENMWmNHZEIhc0VqNVhSUWg=", match[1])

    def test_min_len(self):
        """Test if the min_len parameter works as expected"""
        self.paste.body = "dGVz"
        analyzer = Base64AsciiAnalyzer(None, min_len=4)
        match = analyzer.match(self.paste)
        self.assertTrue(match)

        self.paste.body = "dGVz"
        analyzer = Base64AsciiAnalyzer(None, min_len=5)
        match = analyzer.match(self.paste)
        self.assertFalse(match)

        self.paste.body = "dGVzdFRoaXNTdHJpbmc="
        match = analyzer.match(self.paste)
        self.assertTrue(match)

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        # test that when nothing, is provided nothing matches
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        # test that when nothing, is provided nothing matches
        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # invalid base64 string (% symbol inserted which is not valid base64)
        self.paste.body = "SGVsbG8gV%29ybGQ="
        self.assertFalse(self.analyzer.match(self.paste))

        # not a base64 string
        self.paste.body = "====="
        self.assertFalse(self.analyzer.match(self.paste))

        # base32 encoded string
        self.paste.body = "JBSWY3DPEBLW64TMMQ======"
        self.assertFalse(self.analyzer.match(self.paste))

        # long string (129) not base64
        self.paste.body = "sFm2XgxTt6fuErnWw9JZkae76sL7XDqyNvf2Wkatt9gkzVDxXTf6dCr3Yh6fT82fFzvNWG49P3KSR7XXngHJ5D9ba" \
                          "Dj448rhbNTJrKhRn7TPkYRubZLhmbCrg6bavDa9a"
        self.assertFalse(self.analyzer.match(self.paste))

    def test_invalid_decodes(self):
        """Test to make sure we don't match base64 strings which don't decode to ASCII"""
        # base64 encoded string containing one non-ascii character: "This string contains a non-ascii character: ¤" (UTF-8)
        self.paste.body = "VGhpcyBzdHJpbmcgY29udGFpbnMgYSBub24tYXNjaWkgY2hhcmFjdGVyOiDCpA=="
        self.assertFalse(self.analyzer.match(self.paste))

        # base64 encoded string containing only non-ascii characters: "ΗÈλλθ ωÖΓλÐ" (UTF-8)
        self.paste.body = "zpfDiM67zrvOuCDPicOWzpPOu8OQ"
        self.assertFalse(self.analyzer.match(self.paste))

        # base64 encoded string containing one non-ascii character: "º" (UTF-8)
        self.paste.body = "wro="
        self.assertFalse(self.analyzer.match(self.paste))

    def test_ascii_decode(self):
        """Test if ascii decode flag works"""
        analyzer = Base64AsciiAnalyzer(None, decode=True)

        # base64 encoded string: "Hello World" (UTF-8, LF)
        self.paste.body = "SGVsbG8gV29ybGQ="
        self.assertEqual("Hello World", analyzer.match(self.paste)[0])

        # base64 encoded string: "Hello\nWorld" (UTF-8, LF)
        self.paste.body = "SGVsbG8KV29ybGQ="
        self.assertEqual("Hello\nWorld", analyzer.match(self.paste)[0])

        # base64 encoded string (32 chars): "2fwZ_CTjDKxu48FLCLZcGdB!sEj5XRQh" (UTF-8, LF)
        self.paste.body = "MmZ3Wl9DVGpES3h1NDhGTENMWmNHZEIhc0VqNVhSUWg="
        self.assertEqual("2fwZ_CTjDKxu48FLCLZcGdB!sEj5XRQh", analyzer.match(self.paste)[0])

        # base64 encoded string (64 chars): "Mv=ZH?NJrrBSdhus*KVg%4dG6*C&ub?sSeq!VrzCb_-QcY^KWfxKy8AJ3=^5?b6N" (UTF-8, LF)
        self.paste.body = "TXY9Wkg/TkpyckJTZGh1cypLVmclNGRHNipDJnViP3NTZXEhVnJ6Q2JfLVFjWV5LV2Z4S3k4QUozPV41P2I2Tg=="
        self.assertEqual("Mv=ZH?NJrrBSdhus*KVg%4dG6*C&ub?sSeq!VrzCb_-QcY^KWfxKy8AJ3=^5?b6N", analyzer.match(self.paste)[0])

        # base64 encoded string (256 chars): "etFk!?m@A_vvdMT39Mgcynx_AFz6HY!4R8U3n_7JA?-rF=F3ehWat%4rKfhsuCc98G
        # =t8jMY7hgJDZ2c!y!$!XQATbk6fQD2pa+EdQ_rfP^&_DKJ34dFPcuGjDBTqdxZ&=3U%@dm&?JW#+k@mB%a3TFn%GAzukL+-%TUTq?fAbAKr
        # @y%LPK+KEmxeh+rg7?s3aR2v5A%tbn&_7zNMckCPRd&s8$wW5Bec@aRMCs@4rn?cRx?a&y-Z%kn&h8aLu*R" (UTF-8, LF)
        self.paste.body = "ZXRGayE/bUBBX3Z2ZE1UMzlNZ2N5bnhfQUZ6NkhZITRSOFUzbl83SkE/LXJGPUYzZWhXYXQlNHJLZmhzdUNjO" \
                          "ThHPXQ4ak1ZN2hnSkRaMmMheSEkIVhRQVRiazZmUUQycGErRWRRX3JmUF4mX0RLSjM0ZEZQY3VHakRCVHFkeF" \
                          "omPTNVJUBkbSY/SlcjK2tAbUIlYTNURm4lR0F6dWtMKy0lVFVUcT9mQWJBS3JAeSVMUEsrS0VteGVoK3JnNz9" \
                          "zM2FSMnY1QSV0Ym4mXzd6Tk1ja0NQUmQmczgkd1c1QmVjQGFSTUNzQDRybj9jUng/YSZ5LVola24maDhhTHUqUg=="
        self.assertEqual("etFk!?m@A_vvdMT39Mgcynx_AFz6HY!4R8U3n_7JA?-rF=F3ehWat%4rKfhsuCc98G"
                         "=t8jMY7hgJDZ2c!y!$!XQATbk6fQD2pa+EdQ_rfP^&_DKJ34dFPcuGjDBTqdxZ&=3U%"
                         "@dm&?JW#+k@mB%a3TFn%GAzukL+-%TUTq?fAbAKr@y%LPK+KEmxeh+rg7?s3aR2v5A%tbn&"
                         "_7zNMckCPRd&s8$wW5Bec@aRMCs@4rn?cRx?a&y-Z%kn&h8aLu*R", analyzer.match(self.paste)[0]
                         )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest.mock import Mock, patch

from pastepwn.actions.savefileaction import SaveFileAction


class TestSaveFileAction(unittest.TestCase):

    def setUp(self):
        """Sets up the test case"""
        self.action = SaveFileAction(path="")
        self.paste = Mock()
        self.path_mock = Mock()
        self.path_mock.__truediv__ = Mock(side_effect=self.side_effect)
        self.path_mock.exists = Mock(return_value=True)

    @staticmethod
    def side_effect(path):
        return path

    def test_init(self):
        """Check if parameters are saved correctly"""
        self.action = SaveFileAction(path="/this/is/a/test")
        self.assertEqual("/this/is/a/test", self.action.path.as_posix())

        self.action = SaveFileAction(path="Test")
        self.assertEqual("Test", self.action.path.as_posix())

        # Check if multiple values for init are stored correctly
        self.action = SaveFileAction(path="/test", file_ending=".txt", template="Template string")
        self.assertEqual("/test", self.action.path.as_posix())
        self.assertEqual(".txt", self.action.file_ending)
        self.assertEqual("Template string", self.action.template)

        # Check if
        self.action = SaveFileAction(path="/test")
        self.assertEqual(".txt", self.action.file_ending)
        self.assertEqual("${body}", self.action.template)

    @patch("builtins.open")
    @patch("pastepwn.actions.savefileaction.TemplatingEngine")
    def test_perform_path_not_exists(self, te_mock, open_mock):
        """Check if calling perform with non existing path works as intended."""
        te_mock.fill_template = Mock(return_value="This is some content")

        # Mock pathlib to be independent of filesystem for this test
        self.path_mock.exists = Mock(return_value=False)
        self.action.path = self.path_mock

        # We need to make sure that the file's name is the key
        self.paste.key = "thisIsTheRythmOfTheFile"

        self.action.perform(self.paste, "", [])

        # Since the path didn't exist, we must have called "mkdir" once
        self.action.path.mkdir.assert_called_once_with(parents=True, exist_ok=True)

        # fill_template must be called for the perform method
        te_mock.fill_template.assert_called_once()

        # Make sure that the file was opened and written
        open_mock.assert_called_with("thisIsTheRythmOfTheFile.txt", "w", encoding="utf-8")
        open_mock().__enter__().write.assert_called_with("This is some content")

    @patch("builtins.open")
    @patch("pastepwn.actions.savefileaction.TemplatingEngine")
    def test_perform_path_exists(self, te_mock, open_mock):
        """Check if calling perform with non existing path works as intended."""
        te_mock.fill_template = Mock(return_value="Some other content")

        # Mock pathlib to be independent of filesystem and write rights for this test
        self.action.path = self.path_mock

        # We need to make sure that the file's name is the key
        self.paste.key = "myFile"

        self.action.perform(self.paste, "", [])

        # Since the path did exist, we mustn't call "mkdir"
        self.action.path.mkdir.assert_not_called()

        # fill_template must be called for the perform method
        te_mock.fill_template.assert_called_once()

        # Make sure that the file was opened and written
        open_mock.assert_called_with("myFile.txt", "w", encoding="utf-8")
        open_mock().__enter__().write.assert_called_with("Some other content")

    @patch("builtins.open")
    @patch("pastepwn.actions.savefileaction.TemplatingEngine")
    def test_perform_file_ending_empty(self, te_mock, open_mock):
        """Check if calling perform with non existing path works as intended."""
        te_mock.fill_template = Mock(return_value="Again another content")

        # Mock pathlib to be independent of filesystem and write rights for this test
        self.action.path = self.path_mock
        self.action.file_ending = ""

        # We need to make sure that the file's name is the key
        file_name = "nameOfAFile"
        self.paste.key = Mock()
        self.paste.key.__repr__ = Mock(return_value=file_name)

        self.action.perform(self.paste, "", [])

        # Since the path did exist, we mustn't call "mkdir"
        self.action.path.mkdir.assert_not_called()

        # fill_template must be called for the perform method
        te_mock.fill_template.assert_called_once()

        # We called str(paste.key) once
        self.paste.key.__repr__.assert_called_once()

        # Make sure that the file was opened and written
        open_mock.assert_called_with(file_name, "w", encoding="utf-8")
        open_mock().__enter__().write.assert_called_with("Again another content")

    @patch("builtins.open")
    @patch("pastepwn.actions.savefileaction.TemplatingEngine")
    def test_perform_file_ending(self, te_mock, open_mock):
        """Check if calling perform with non existing path works as intended."""
        te_mock.fill_template = Mock(return_value="This is some content")

        self.action.path = self.path_mock
        self.action.file_ending = ".asdf"

        # We need to make sure that the file's name is the key
        self.paste.key = Mock()
        self.paste.key.__repr__ = Mock(return_value="123456")

        self.action.perform(self.paste, "", [])

        self.action.path.mkdir.assert_not_called()
        te_mock.fill_template.assert_called_once()
        self.paste.key.__repr__.assert_called_once()

        # Make sure that the file was opened and written
        open_mock.assert_called_with("123456.asdf", "w", encoding="utf-8")
        open_mock().__enter__().write.assert_called_with("This is some content")

    @patch("pastepwn.actions.savefileaction.TemplatingEngine")
    def test_get_file_content(self, te_mock):
        """Check if the content of the file is returned correctly"""
        te_mock.fill_template = Mock(return_value="This is the content")
        content = self.action.get_file_content(self.paste, "", [])
        self.assertEqual("This is the content", content)

    def test__remove_prefix(self):
        """Check if removing prefixes from a string works fine."""
        input_string = ".txt"
        prefix = "."
        res = self.action._remove_prefix(input_string, prefix)
        self.assertEqual("txt", res)

        input_string = ".json"
        prefix = "."
        res = self.action._remove_prefix(input_string, prefix)
        self.assertEqual("json", res)

        input_string = ".txt"
        prefix = "_"
        res = self.action._remove_prefix(input_string, prefix)
        self.assertEqual(".txt", res)

        input_string = ""
        prefix = "."
        res = self.action._remove_prefix(input_string, prefix)
        self.assertEqual("", res)

        input_string = "..txt"
        prefix = "."
        res = self.action._remove_prefix(input_string, prefix)
        self.assertEqual(".txt", res)

        input_string = ".txt."
        prefix = "."
        res = self.action._remove_prefix(input_string, prefix)
        self.assertEqual("txt.", res)

        input_string = "This is a very long string without prefix"
        prefix = "A"
        res = self.action._remove_prefix(input_string, prefix)
        self.assertEqual("This is a very long string without prefix", res)

        input_string = "AAABBBCCC"
        prefix = "A"
        res = self.action._remove_prefix(input_string, prefix)
        self.assertEqual("AABBBCCC", res)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class BattleNetKeyAnalyzer(RegexAnalyzer):
    """Analyzer to match battle.net keys via regex."""
    name = "BattleNetKeyAnalyzer"

    def __init__(self, actions):
        # battle.net activation page shows 4-4-4-4 format, but there are also codes in the 4-4-5-4-4 format which work
        regex = r"\b(?<!-)([A-Z0-9]{4}\-[A-Z0-9]{4}\-[A-Z0-9]{4}\-[A-Z0-9]{4}|[A-Z0-9]{4}\-[A-Z0-9]{4}\-[A-Z0-9]{5}\-[A-Z0-9]{4}\-[A-Z0-9]{4})\b(?!-)"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class MegaLinkAnalyzer(RegexAnalyzer):
    """Analyzer to match mega links via regex."""
    name = "MegaLinkAnalyzer"

    def __init__(self, actions):
        regex = r"\b(?:https?:\/\/)?mega\.nz\/#F![\w-]{8}(?:![\w-]{22}(?:![\w-]{8})?)?\b"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
from .logicalbaseanalyzer import LogicalBaseAnalyzer


class AndAnalyzer(LogicalBaseAnalyzer):
    """Meta analyzer which matches a paste if all of the passed analyzers match that paste"""
    name = "AndAnalyzer"

    def match(self, paste):
        """Returns True if all of the passed analyzers matched"""
        if not self.analyzers:
            return False

        for analyzer in self.analyzers:
            if not analyzer.match(paste):
                return False

        return True



# -*- coding: utf-8 -*-
import logging

from pastepwn.actions import BasicAction
from pastepwn.errors import InvalidActionError
from pastepwn.util import listify


class BasicAnalyzer(object):
    """Basic analyzer class"""
    name = "BasicAnalyzer"

    def __init__(self, actions, identifier=None):
        """Basic analyzer which is extended to create other analyzer subclasses.
        :param actions: A single action or a list of actions to be executed on every paste
        :param identifier: The name or unique identifier for this specific analyzer
        """
        self.logger = logging.getLogger(__name__)
        self.actions = listify(actions)
        self.identifier = identifier or self.name

        # Check if passed action is an instance of an analyzer and not a class - see #175
        # Raises an error if any action is not an object inheriting from BasicAaction
        for action in self.actions:
            self._check_action(action)

    def add_action(self, action):
        """Adds a new action to the already present actions.
        :param action: New action to add to the present actions
        :return: None
        """
        self._check_action(action)
        self.actions.append(action)

    def match(self, paste, unique_matches=False):
        """Checks if a certain paste is matched by the conditions set for this analyzer.
        :param paste: A Paste object which should be matched
        :param unique_matches: A boolean that specifies if matches should be unique - has currently NO effect on merged analyzers!
        :return: A boolean that indicates if the paste has been matched
        """
        raise NotImplementedError("Your analyzer must implement the match method!")

    @staticmethod
    def unique(matches):
        """Takes a list of matches and returns a list with no duplicates.
        :param matches: A list of matches
        :return: A filtered list of matches retaining order
        """
        return sorted(set(matches), key=matches.index)

    def _check_action(self, action):
        """Check if a passed action is a subclass of BasicAction"""
        if not isinstance(action, BasicAction):
            if isinstance(action, type):
                error_msg = "You passed a class as action for '{}' but an instance of an action was expected!".format(self.identifier)
            else:
                error_msg = "You did not pass an action object - inheriting from BasicAction - to '{}'".format(self.identifier)

            self.logger.error(error_msg)
            raise InvalidActionError(error_msg)

    def __and__(self, other):
        return MergedAnalyzer(self, and_analyzer=other)

    def __or__(self, other):
        return MergedAnalyzer(self, or_analyzer=other)

    def __repr__(self):
        if self.identifier is None:
            self.identifier = self.__class__.__name__
        return self.identifier


class MergedAnalyzer(BasicAnalyzer):
    """Combination class to combine multiple analyzers into a single one.
    Doesn't need to be created manually - use the binary operators (& and |) to combine multiple analyzers.
    """
    name = "MergedAnalyzer"

    def __init__(self, base_analyzer, and_analyzer=None, or_analyzer=None):
        self._base_analyzer = base_analyzer
        self._and_analyzer = and_analyzer
        self._or_analyzer = or_analyzer

        if self._and_analyzer:
            actions = base_analyzer.actions + self._and_analyzer.actions
            identifier = "({} && {})".format(base_analyzer.identifier, self._and_analyzer)
        elif self._or_analyzer:
            actions = base_analyzer.actions + self._or_analyzer.actions
            identifier = "({} || {})".format(base_analyzer.identifier, self._or_analyzer)
        else:
            raise ValueError("Neither and_analyzer nor or_analyzer are set!")

        super().__init__(actions, identifier=identifier)

    def match(self, paste, unique_matches=False):
        """Checks if a certain paste is matched by the conditions set for this analyzer.
        :param paste: A Paste object which should be matched
        :param unique_matches: A boolean that specifies if matches should be unique - has currently NO effect on merged analyzers!
        :return: A boolean that indicates if the paste has been matched
        """
        base_analyzer_match = bool(self._base_analyzer.match(paste))
        if self._and_analyzer:
            return base_analyzer_match and bool(self._and_analyzer.match(paste))
        elif self._or_analyzer:
            return base_analyzer_match or bool(self._or_analyzer.match(paste))



# -*- coding: utf-8 -*-
import logging
import os
import sys
import unittest
from unittest.mock import Mock

from pastepwn import Paste
from pastepwn.actions.discordaction import DiscordAction


@unittest.skipIf(os.getenv("CI"), "running on CI")
class TestDiscordAction(unittest.TestCase):
    def setUp(self):
        """Setup the environment to test the action"""
        pass

    def tearDown(self):
        """Reset the environment to a clean state"""
        pass

    @staticmethod
    def generate_paste():
        """Returns a paste for testing"""
        p = {"scrape_url": "https://scrape.pastebin.com/api_scrape_item.php?i=0CeaNm8Y",
             "full_url": "https://pastebin.com/0CeaNm8Y",
             "date": "1442911802",
             "key": "0CeaNm8Y",
             "size": "890",
             "expire": "1442998159",
             "title": "Once we all know when we goto function",
             "syntax": "java",
             "user": "admin",
             "body": "This is a test for pastepwn"}

        return Paste(p.get("key"), p.get("title"), p.get("user"), p.get("size"), p.get("date"), p.get("expire"), p.get("syntax"), p.get("scrape_url"),
                     p.get("full_url"))

    def test_init(self):
        """Check if initializing the action sets it up correctly"""
        webhook = Mock()
        token = Mock()
        channel_id = Mock()
        template = Mock()

        self.logger = logging.getLogger("pastepwn.actions.discordaction")

        with self.assertLogs(self.logger, level="WARNING") as log:
            # Make sure that the import is not present!
            sys.modules["websockets"] = None
            action = DiscordAction(webhook_url=webhook, token=token, channel_id=channel_id, template=template)
            self.assertEqual(log.output,
                             ["WARNING:pastepwn.actions.discordaction:Could not import 'websockets' module. So you can only use webhooks for discord."])

        self.assertEqual(action.webhook_url, webhook)
        self.assertEqual(action.template, template)
        self.assertFalse(action.bot_available, msg="Websockets are not available. bot_available must be false!")

    def test_init_logging(self):
        """Make sure that the action logs information about missing imports"""
        webhook = Mock()
        token = Mock()
        channel_id = Mock()
        template = Mock()

        self.logger = logging.getLogger("pastepwn.actions.discordaction")

        with self.assertLogs(self.logger, level="WARNING") as log:
            # Make sure that the import is not present, so that we do log!
            sys.modules["websockets"] = None
            action = DiscordAction(webhook_url=webhook, token=token, channel_id=channel_id, template=template)
            self.assertEqual(log.output,
                             ["WARNING:pastepwn.actions.discordaction:Could not import 'websockets' module. So you can only use webhooks for discord."])

        self.assertEqual(action.webhook_url, webhook)
        self.assertEqual(action.template, template)
        self.assertFalse(action.bot_available, msg="Websockets are not available. bot_available must be false!")

    def test_init_import_successful(self):
        """Check if the initialization works fine if the import is successful"""
        webhook = Mock()
        token = Mock()
        channel_id = Mock()
        template = Mock()

        self.logger = logging.getLogger("pastepwn.actions.discordaction")

        # Make sure that the import is present!
        sys.modules["websockets"] = Mock()
        action = DiscordAction(webhook_url=webhook, token=token, channel_id=channel_id, template=template)
        self.assertTrue(action.bot_available, msg="Websockets are available. bot_available must be true!")

        # We are using the webhooks so we don't have the token/channel_id attribute
        self.assertEqual(action.webhook_url, webhook)
        self.assertEqual(action.template, template)

    def test_init_token(self):
        """Check if initializing the action with a token works as expected"""
        token = Mock()
        channel_id = Mock()
        template = Mock()

        sys.modules["websockets"] = Mock()
        action = DiscordAction(webhook_url=None, token=token, channel_id=channel_id, template=template)

        self.assertEqual(action.token, token)
        self.assertEqual(action.channel_id, channel_id)
        self.assertEqual(action.template, template)
        self.assertTrue(action.bot_available)
        self.assertFalse(action.identified)

    def test_init_token_exception(self):
        """Check if initializing the action with a token but without websockets module throws an exception"""
        token = Mock()
        channel_id = Mock()
        template = Mock()

        sys.modules["websockets"] = None
        with self.assertRaises(NotImplementedError, msg="No exception although module could not be imported"):
            # When it's not possible to import the websockets module, we need to throw an exception
            _ = DiscordAction(webhook_url=None, token=token, channel_id=channel_id, template=template)

    def test_init_general_exception(self):
        """Check if an exception occurrs on an error state"""
        with self.assertRaises(ValueError, msg="No error raised, although both token and channel_id are None!"):
            _ = DiscordAction(webhook_url=None, token=None, channel_id=None)

        with self.assertRaises(ValueError, msg="No error raised, although channel_id is None!"):
            _ = DiscordAction(webhook_url=None, token=Mock(), channel_id=None)

        with self.assertRaises(ValueError, msg="No error raised, although token is None!"):
            _ = DiscordAction(webhook_url=None, token=None, channel_id=Mock())


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.mailchimpapikeyanalyzer import MailChimpApiKeyAnalyzer


class TestMailChimpApiKeyAnalyzer(unittest.TestCase):

    def setUp(self):
        self.analyzer = MailChimpApiKeyAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        self.paste.body = "87b6ac876aca87c687a6c87a6ca876c8-us000000000000"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa-us000000000000"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "12312312312312312312312312312312-us000000000000"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa-us345646456456"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "asdasdasdasdasdasdasdasdasdasdas-us000000000000"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa-it000000000000"
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa-usasdasdasdasd"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.adobekeyanalyzer import AdobeKeyAnalyzer


class TestAdobeKeyAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = AdobeKeyAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # adobe key dump
        self.paste.body = "1118-1993-2045-6322-6067-9110"
        self.assertTrue(self.analyzer.match(self.paste))
        self.assertEqual("1118-1993-2045-6322-6067-9110", self.analyzer.match(self.paste)[0])

        # adobe key dump
        self.paste.body = "1118-1551-7298-8490-8910-4435"
        self.assertTrue(self.analyzer.match(self.paste))
        self.assertEqual("1118-1551-7298-8490-8910-4435", self.analyzer.match(self.paste)[0])

        # adobe key dump
        self.paste.body = "1118-1088-9818-3636-2479-0297"
        self.assertTrue(self.analyzer.match(self.paste))
        self.assertEqual("1118-1088-9818-3636-2479-0297", self.analyzer.match(self.paste)[0])

        # adobe key dump
        self.paste.body = "1118-1194-1581-4556-8113-6593"
        self.assertTrue(self.analyzer.match(self.paste))
        self.assertEqual("1118-1194-1581-4556-8113-6593", self.analyzer.match(self.paste)[0])

        # part of a sentence
        self.paste.body = "Hey, I have your key right here: 1118-1470-8779-0264-4009-3244!"
        self.assertTrue(self.analyzer.match(self.paste))
        self.assertEqual("1118-1470-8779-0264-4009-3244", self.analyzer.match(self.paste)[0])

        # Newline seperated microsoft key
        self.paste.body = "1118-1993-2046-6322-6067-9110\n1118-1470-8779-0264-4009-3244"
        self.assertTrue(self.analyzer.match(self.paste))
        self.assertEqual("1118-1993-2046-6322-6067-9110", self.analyzer.match(self.paste)[0])
        self.assertEqual("1118-1470-8779-0264-4009-3244", self.analyzer.match(self.paste)[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "1118-1470-8779-0264-4009-32445"
        self.assertFalse(self.analyzer.match(self.paste))
        self.assertEqual(list, type(self.analyzer.match(self.paste)))

        # Invalid length
        self.paste.body = "1118-1470-8779-0264-4009-324"
        self.assertFalse(self.analyzer.match(self.paste))
        self.assertEqual(list, type(self.analyzer.match(self.paste)))

        # Invalid Characters
        self.paste.body = "1118-1194-1581-4556-8113-659A"
        self.assertFalse(self.analyzer.match(self.paste))
        self.assertEqual(list, type(self.analyzer.match(self.paste)))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from pastepwn.analyzers.basicanalyzer import BasicAnalyzer
from pastepwn.util import listify


class LogicalBaseAnalyzer(BasicAnalyzer):
    """Meta analyzer used to combine analyzers via a logical operator"""
    name = "LogicalBaseAnalyzer"

    def __init__(self, actions, analyzers, merge_actions=False):
        """
        Meta analyzer used to combine analyzers via a logical operator
        :param actions: A single action or a list of actions to be executed on every paste
        :param analyzers: A single analyzer or a list of analyzers used to match pastes
        """
        super().__init__(actions)
        self.analyzers = listify(analyzers)

        if not self.analyzers:
            self.logger.warning("You have not specified any analyzers inside '{}'".format(self.name))

        if merge_actions:
            self._merge_actions()

    def _merge_actions(self):
        """
        Merges the actions of the passed analyzers into this meta analyzer
        If merged, the actions will be executed if this meta analyzer matches
        :return: None
        """
        for analyzer in self.analyzers:
            for action in analyzer.actions:
                self.actions.append(action)

    def add_analyzer(self, analyzer):
        """
        Add a new analyzer to the list of analyzers
        :param analyzer: A single analyzer used to match pastes
        :return:
        """
        self.analyzers.append(analyzer)

    def match(self, paste):
        """Must be overridden by the subclasses"""
        raise NotImplementedError("match() function must be overridden in the subclasses!")



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class FacebookAccessTokenAnalyzer(RegexAnalyzer):
    """Analyzer to match a Facebook Access Token"""
    name = "FacebookAccessTokenAnalyzer"
    regex = r"EAACEdEose0cBA[0-9A-Za-z]+"

    def __init__(self, actions):
        """
        Analyzer which always matches a paste to perform actions on every paste
        :param actions: A single action or a list of actions to be executed on every paste
        """
        super().__init__(actions, self.regex)



# -*- coding: utf-8 -*-
import logging
import socket

import requests.packages.urllib3.util.connection as urllib3_cn


def _allowed_gai_family_4():
    """https://github.com/urllib3/urllib3/blob/master/src/urllib3/util/connection.py"""
    return socket.AF_INET


def _allowed_gai_family_6():
    """https://github.com/urllib3/urllib3/blob/master/src/urllib3/util/connection.py"""
    return socket.AF_INET6


def _allowed_gai_family_unspec():
    """https://github.com/urllib3/urllib3/blob/master/src/urllib3/util/connection.py"""
    return socket.AF_UNSPEC


def enforce_ip_version(version=None):
    """
    Tries to enforce either IPv4 or IPv6, depending on the version parameter
    :param version: The IP version to use
    :return:
    """
    if version is None:
        urllib3_cn.allowed_gai_family = _allowed_gai_family_unspec
        return

    version = str(version)

    if version == "4":
        logging.info("Enforcing IPv4!")
        urllib3_cn.allowed_gai_family = _allowed_gai_family_4

    elif version == "6":
        logging.info("Enforcing IPv6!")
        if not urllib3_cn.HAS_IPV6:
            raise Exception("Your system can't handle IPv6!")

        urllib3_cn.allowed_gai_family = _allowed_gai_family_6

    elif version == "5":
        raise ValueError("Internet Stream Protocol? Really? There is no actual IPv5!")

    else:
        raise ValueError("No valid value specified for 'version' parameter! Please use either 4 or 6")



# -*- coding: utf-8 -*-
import re
import urllib.error
import urllib.request

from .basicanalyzer import BasicAnalyzer


class URLAnalyzer(BasicAnalyzer):
    """This analyzer detects url patterns using regex, and optionally attempts to verify that they can be resolved."""

    name = "URLAnalyzer"

    def __init__(self, actions, regex, resolve=False):
        super().__init__(actions, regex)
        self.regex = re.compile(regex)
        self.resolve = resolve  # Should we try to resolve the URLs?

    def _resolve_url(self, url):
        """A helper method that tries to resolve a given URL.
        Returns False if the URL cannot be resolved.
        """
        # If the url doesn't start with a protocol, we'll test with http and https.
        if not url.lower().startswith("http"):
            for protocol in ("http", "https"):
                # If adding the protocol makes this resolve, the url works.
                if self._resolve_url("{0}://{1}".format(protocol, url)):
                    return True
            return False

        # Otherwise, let's just try resolving it.
        try:
            urllib.request.urlopen(url)
        except urllib.error.URLError:
            return False

        return True

    def match(self, paste):
        """Check if the URL matches a certain regex, and then optionally try to resolve it."""
        paste_content = paste.body or ""
        match = self.regex.search(paste_content)
        is_regex = match is not None

        # Optionally check if the url resolves.
        if match and self.resolve:
            url = match.group(0)
            return self._resolve_url(url)

        return is_regex



# -*- coding: utf-8 -*-
import os

from setuptools import setup, find_packages

setup_path = os.path.dirname(os.path.abspath(__file__))
packages = find_packages(exclude=["tests*"])


# Taken from https://github.com/python-telegram-bot/python-telegram-bot/blob/9d99660ba95b103b3e1dc80414a5ce2fd805260b/setup.py#L9
def requirements():
    """Build the requirements list for this project"""
    requirements_list = []

    with open(os.path.join(setup_path, "requirements_minimum.txt")) as reqs:
        for install in reqs:
            requirements_list.append(install.strip())

        return requirements_list


with open(os.path.join(setup_path, "README.md"), "r", encoding="utf-8") as file:
    readme = file.read()

# Check if we are running on CI
CI = os.environ.get("CI")

if CI:
    version = ""
    TRAVIS_TAG = os.environ.get("TRAVIS_TAG")
    GITHUB_ACTIONS = os.environ.get("GITHUB_ACTIONS")

    if TRAVIS_TAG:
        print("Running on Travis!")
        version = TRAVIS_TAG.replace("v", "")
    elif GITHUB_ACTIONS:
        print("Running on GitHub Actions!")
        GITHUB_REF = os.environ.get("GITHUB_REF")
        tag = GITHUB_REF.split("/")[-1]
        version = tag.replace("v", "")
else:
    # Taken from https://packaging.python.org/guides/single-sourcing-package-version/
    version_dict = {}
    version_file = os.path.join(setup_path, "pastepwn", "version.py")
    with open(version_file, "r", encoding="utf-8") as file:
        exec(file.read(), version_dict)
    version = version_dict["__version__"]

print("Building version {} of pastepwn".format(version))

setup(name="pastepwn",
      version=version,
      install_requires=requirements(),
      keywords="python pastebin scraping osint framework",
      description="Python framework to scrape PasteBin pastes and analyze them",
      long_description=readme,
      long_description_content_type="text/markdown",
      url="https://github.com/d-Rickyy-b/pastepwn",
      author="d-Rickyy-b",
      author_email="pastepwn@rico-j.de",
      license="MIT",
      packages=packages,
      include_package_data=True,
      classifiers=[
          "Development Status :: 5 - Production/Stable",
          "Environment :: Console",
          "Intended Audience :: Developers",
          "Intended Audience :: Science/Research",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
          "Topic :: Software Development :: Libraries :: Python Modules",
          "Topic :: Security",
          "Topic :: Internet",
          "Programming Language :: Python",
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
          "Programming Language :: Python :: 3.8",
          "Programming Language :: Python :: 3.9",
          ],
      )



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.originkeyanalyzer import OriginKeyAnalyzer


class TestOriginKeyAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = OriginKeyAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # origin key dump
        self.paste.body = "87NN-Z277-KTHD-EDJL-7MVB"
        self.assertTrue(self.analyzer.match(self.paste))

        # origin key dump
        self.paste.body = "DPMV-FZXD-GD2G-7U28-6S3D"
        self.assertTrue(self.analyzer.match(self.paste))

        # origin key dump
        self.paste.body = "ACT3-3TD6-ZDYB-BCGP-TTDM"
        self.assertTrue(self.analyzer.match(self.paste))

        # origin key dump
        self.paste.body = "63WW-8VQE-7VEA-HFCD-LVYT"
        self.assertTrue(self.analyzer.match(self.paste))

        # part of a sentence
        self.paste.body = "Hey, I have your key right here: 87NN-Z277-KTHD-EDJL-7MVB!"
        self.assertTrue(self.analyzer.match(self.paste))

        # Newline seperated origin key
        self.paste.body = "87NN-Z277-KTHD-EDJL-7MVB\nDPMV-FZXD-GD2G-7U28-6S3D"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("87NN-Z277-KTHD-EDJL-7MVB", match[0])
        self.assertEqual("DPMV-FZXD-GD2G-7U28-6S3D", match[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid length
        self.paste.body = "A63WW-8VQE-7VEA-HFCD-LVYT"
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid Characters
        self.paste.body = "63W_-8VQE-7VEA-HFCD-LVYT"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.hashanalyzer import HashAnalyzer


class TestHashAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = HashAnalyzer(None, [
            b"hunter2",
            b"SuperSecurePassword!123",
            b""
            ])
        self.paste = mock.Mock()

    def test_match(self):
        md5_hashes = ["2ab96390c7dbe3439de74d0c9b0b1767",
                      "0fceb0a9f3d9e4a454df92965c7d9f3e",
                      "d41d8cd98f00b204e9800998ecf8427e"
                      ]
        sha1_hashes = ["f3bbbd66a63d4bf1747940578ec3d0103530e21d",
                       "1a0f39c3bdc08e7e3bb6f02ed181ad96ee90f766",
                       "da39a3ee5e6b4b0d3255bfef95601890afd80709"
                       ]
        sha256_hashes = ["f52fbd32b2b3b86ff88ef6c490628285f482af15ddcb29541f94bcf526a3f6c7",
                         "4bf33443cc946e7ef2b394f6874a74113c7b7b142765afcdfca142ad32a1bf1e",
                         "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855"
                         ]

        invalid_hashes = ["9f9d51bc70ef21ca5c14f307980a29d8",
                          "5f4dcc3b5aa765d61d8327deb882cf99",
                          "81b637d8fcd2c6da6359e6963113a1170de795e4b725b84d1e0b4cfd9ec58ce9",
                          "5e884898da28047151d0e56f8dc6292773603d0d6aabbdd62a11ef721d1542d8"
                          ]

        for test_hash in (md5_hashes + sha1_hashes + sha256_hashes):
            self.paste.body = test_hash
            self.assertTrue(self.analyzer.match(self.paste), test_hash)

        for test_hash in invalid_hashes:
            self.paste.body = test_hash
            self.assertFalse(self.analyzer.match(self.paste), test_hash)

    def test_match_case_insensitive(self):
        self.paste.body = "2ab96390c7dbe3439de74d0c9b0b1767"
        self.assertTrue(self.analyzer.match(self.paste), "lowercase hash")
        self.paste.body = "2AB96390C7DBE3439DE74D0C9B0B1767"
        self.assertTrue(self.analyzer.match(self.paste), "uppercase hash")

    def test_match_md5_only(self):
        md5_analyzer = HashAnalyzer(None, [b"hunter2", b"SuperSecurePassword!123", b""], ["md5"])
        self.paste.body = "0fceb0a9f3d9e4a454df92965c7d9f3e"
        self.assertTrue(md5_analyzer.match(self.paste), "MD5 hash")
        self.paste.body = "1a0f39c3bdc08e7e3bb6f02ed181ad96ee90f766"
        self.assertFalse(md5_analyzer.match(self.paste), "SHA-1 hash")

    def test_unavailable_algorithms(self):
        with self.assertRaises(ValueError):
            HashAnalyzer(None, [b"hunter2"], ["super_algorithm_not_available"])

    def test_single_password_input(self):
        single_password_analyzer = HashAnalyzer(None, b"hunter2")
        self.paste.body = "f52fbd32b2b3b86ff88ef6c490628285f482af15ddcb29541f94bcf526a3f6c7"
        self.assertTrue(single_password_analyzer.match(self.paste), "SHA-256 hash")
        self.paste.body = "aaa9402664f1a41f40ebbc52c9993eb66aeb366602958fdfaa283b71e64db123"
        self.assertFalse(single_password_analyzer.match(self.paste), "SHA-256 hash of 'h'")



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class AWSAccessKeyAnalyzer(RegexAnalyzer):
    """Analyzer to match AWS Access Key via regex.

    Keys are 20 character alphanumeric /+=
    """
    name = "AWSAccessKeyAnalyzer"

    def __init__(self, actions):
        # https://people.eecs.berkeley.edu/~rohanpadhye/files/key_leaks-msr15.pdf
        regex = r"\b(?:A3T[A-Z0-9]|AKIA|AGPA|AROA|AIPA|ANPA|ANVA|ASIA)[A-Z0-9]{16}\b"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
import logging
import select
import socket
from queue import Empty, Queue
from threading import Event
from time import sleep

from pastepwn.util import TemplatingEngine
from pastepwn.util.threadingutils import start_thread, join_threads
from .basicaction import BasicAction

MAX_MSG_SIZE = 512


class IrcAction(BasicAction):
    """Action to send an irc message to a certain channel"""
    name = "IrcAction"

    def __init__(self, server=None, channel=None, port=6667, nick="pastepwn", template=None):
        super().__init__()
        self.logger = logging.getLogger(__name__)
        self._msg_queue = Queue()
        self.ircsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.connected = False

        self.server = server
        self.port = port
        self.nick = nick
        self.template = template

        if not channel.startswith("#"):
            channel = "#{0}".format(channel)
        self.channel = channel

        # RFC1459 says that each message can only be 512 bytes including the CR-LF character
        self._max_payload_size = MAX_MSG_SIZE - len("PRIVMSG {0}".format(channel)) - len("\r\n")

        self._exception_event = Event()
        self._stop_event = Event()

        self.logger.info("Starting up the IRC client - pastepwn can only send messages, as soon as the client is ready!")
        self._thread = start_thread(self._run_irc_client, "irc_client", exception_event=self._exception_event)

    def _run_irc_client(self):
        """Runs an IRC client, which handles sending messages and answering on Server PINGs.
        :return: None
        """
        # Connect to the IRC
        self._connect()
        self._login()
        self._join()

        while True:
            # Reference: https://hardmath123.github.io/socket-science-2.html
            data = ""
            readables, writables, exceptionals = select.select([self.ircsock], [self.ircsock], [self.ircsock])
            # When we got a readable socket, we read it's content
            if len(readables) == 1:
                data += self.ircsock.recv(MAX_MSG_SIZE).decode("UTF-8")

                if data == 0:
                    self.logger.error("The socket was disconnected!")
                    self._reconnect()

                # Split up the data in single IRC messages to handle them separately
                while "\r\n" in data:
                    message = data[:data.index("\r\n")]
                    data = data[data.index("\r\n") + 2:]
                    self._handle_message(message)

            # We use the _stop_event to kill our thread
            if self._stop_event.is_set():
                break

            # As long as we are not connected, we don't want to send any messages
            if not self.connected:
                continue

            try:
                # We don't need to speed up message sending - We are only allowed 1 message every 2 seconds according to RFC 1459
                msg = self._msg_queue.get(True, 1)
            except Empty:
                continue

            self.logger.debug("New message on msg_queue: {0}".format(msg))
            self._send("PRIVMSG {0} :{1}".format(self.channel, msg))

    def _handle_message(self, message):
        """
        Handles IRC server messages
        :param message: A messare received from the IRC server
        :return:
        """
        self.logger.debug("Server message: {}".format(message))
        words = message.split(" ")

        if message.startswith("PING"):
            self._pong()

        elif words[1] == "020" and not self.connected:
            # We connected our socket and now need to send our nick
            self._login()

        elif words[1] == "001" and not self.connected:
            # The server accepted our nick and sent us their MOTD. We now join a channel
            self.connected = True
            self._join()

        elif words[1] == "PRIVMSG" and words[2] == self.channel and "!status" in words[3] and self.connected:
            # A little gimmick to check the status of pastepwn via IRC
            self._send_message("Pastepwn is still functional and operating!")

    def _send(self, data):
        """
        Sends data to the IRC server over a socket
        :return: None
        """
        try:
            self.ircsock.send(bytes("{0}\r\n".format(data), "UTF-8"))
        except ConnectionAbortedError as e:
            self.logger.error("Connection to IRC server lost: ", e)
            self._reconnect()

    def _reconnect(self):
        """
        Tries to reconnect to the IRC Server
        :return:
        """
        self.connected = False
        self._exception_event.set()

        while True:
            try:
                self._connect()
            except Exception as e:
                self.logger.error("Exception while trying to connect to the IRC server occurred: ", e)
                self.logger.info("Sleeping for 10 seconds before trying next reconnect!")
                sleep(10)
            else:
                # No exception occurred - Our socket should be connected again
                return

    def _connect(self):
        """
        Connect to the IRC Server
        :return: None
        """
        self.logger.debug("Connecting to IRC server '{}:{}' using nick {}.".format(self.server, self.port, self.nick))
        self.ircsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.ircsock.connect((self.server, self.port))

    def _login(self):
        """
        Send login data to the IRC server
        :return:
        """
        self._send("NICK {}".format(self.nick))
        self._send("USER {} 8 * :{}".format(self.nick, self.nick))

    def _join(self):
        """
        Joins an IRC channel
        :return: None
        """
        self.logger.debug("Joining channel '{}' on '{}:{}' using nick {}.".format(self.channel, self.server, self.port, self.nick))
        self._send("JOIN {}".format(self.channel))

    def _quit(self, msg=None):
        """
        Quit from an IRC server
        :return: None
        """
        self.logger.info("Sending QUIT message to the server '{}:{}' from nick '{}'".format(self.server, self.port, self.nick))
        if msg is not None and msg != "":
            msg = msg.replace("\r\n", " ").replace("\r", " ").replace("\n", " ")
            self._send("QUIT :{}".format(msg))
        else:
            self._send("QUIT")

    def _pong(self):
        """
        Reply to a server PING with PONG
        :return: None
        """
        self.logger.debug("Server PING received. Replying to server with PONG.")
        self._send("PONG :{}".format(self.nick))

    def _send_message(self, msg):
        """
        Sends a message to the specified IRC channel by putting it on the msg_queue
        Maximum sending frequency is 1 message every 2 seconds (according to RFC 1459)
        :param msg: String containing a message
        :return:
        """
        # We need to remove all newlines in a message, so that the whole message can be sent
        # because RFC 1459 defines CRLF as End of Message
        msg = msg.replace("\r\n", " ")
        msg = msg.replace("\r", " ")
        msg = msg.replace("\n", " ")
        if len(msg) > self._max_payload_size:
            # We need to split up the message into two parts and send it recursively
            self._send_message(msg[:self._max_payload_size])
            self._send_message(msg[self._max_payload_size:])
            return

        # Otherwise we can simply put it on the queue as a whole
        self._msg_queue.put(msg)

    def __del__(self):
        self._stop_event.set()
        join_threads([self._thread])

    def perform(self, paste, analyzer_name=None, matches=None):
        """Perform the action on the passed paste"""
        if self._exception_event.is_set():
            self.logger.error("The exception event is set. The IRC action might not perform as it should! Messages will"
                              " be buffered for the case of a reconnect.")
        text = TemplatingEngine.fill_template(paste, analyzer_name, template_string=self.template, matches=matches)
        self._send_message(text)



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class SlackWebhookAnalyzer(RegexAnalyzer):
    """Analyzer to match (likely) Slack Webhook URLs"""

    def __init__(self, action):
        """
        Analyzer to match (likely) Slack Webhook URLs
        :param action: Single action or list of actions to be executed when a paste matches
        """
        regex = r"https://hooks.slack.com/services/T[a-zA-Z0-9_]{8}/B[a-zA-Z0-9_]{8}/[a-zA-Z0-9_]{24}"

        super().__init__(action, regex)



# -*- coding: utf-8 -*-
import json
import logging
import time

from pastepwn.util import Request
from .basicaction import BasicAction


class MISPAction(BasicAction):
    """
    Action to add an event to a MISP instance on a matched paste

    Documentation for adding events:
    https://www.circl.lu/doc/misp/automation/#post-events

    The MISPAction objects can take a `transformer` function as a constructor parameter.
    This function (by default MISPAction.default_transformer) should take a Paste and an
    optional analyzer name as parameters (just like BasicAction.perform), and return a
    dictionary representing a MISP event, which will then be sent to the API.

    Additional attributes can be sent with each event, specified by the `attributes`
    parameter. Here is the documentation regarding types and categories:
    https://www.circl.lu/doc/misp/categories-and-types/
    """
    name = "MISPAction"

    def __init__(self, url, access_key, transformer=None, attributes=None):
        """Init method for the MISPAction
        :param url:         string      URL of the MISP instance (complete with protocol and port)
        :param access_key:  string      MISP access key for authorization
        :param transformer: Callable    Takes a Paste (and optional analyzer name) as parameter
                                        and returns a MISP-formatted event as a dictionary
        :param attributes:  Iterable    List of fully defined attributes to add to events
        """
        super().__init__()
        self.logger = logging.getLogger(__name__)
        self.url = url
        self.access_key = access_key
        if transformer is None:
            self.transformer = MISPAction.default_transformer
        else:
            self.transformer = transformer
        self.attributes = attributes

    @staticmethod
    def default_transformer(paste, analyzer_name=None):
        timestamp = time.gmtime(int(paste.date))
        attrs = []
        # Build event
        event = {
            "date": time.strftime("%Y-%m-%d", timestamp),
            "info": "Sensitive information found on pastebin (type: %s)" % analyzer_name,
            "threat_level_id": 4,   # Undefined
            "published": False,     # Unpublished
            "analysis": 0,          # Not yet analyzed
            "distribution": 0,      # Shared with organization only
            "Attribute": []
            }
        # Add link to the paste
        attrs.append({
            "type": "url",
            "category": "Network activity",
            "comment": "Link to pastebin paste containing information",
            "value": paste.full_url
            })
        # Add username of the author
        attrs.append({
            "type": "text",
            "category": "Attribution",
            "comment": "Username of paste author",
            "value": paste.user
            })
        # Add size of the paste
        attrs.append({
            "type": "size-in-bytes",
            "category": "Other",
            "comment": "Size of the paste",
            "value": paste.size
            })
        # Attach full paste if it's small
        if int(paste.size) <= 1024 and paste.body is not None:
            attrs.append({
                "type": "attachment",
                "category": "Artifacts dropped",
                "comment": "Raw body of the paste",
                "value": paste.body
                })
        # Add attributes to the event
        event["Attribute"] = attrs
        return event

    def perform(self, paste, analyzer_name=None, matches=None):
        """
        Sends the event to the MISP instance.
        :param paste: The paste passed by the ActionHandler
        :param analyzer_name: The name of the analyzer which matched the paste
        """
        # Call transformer to construct payload
        event = self.transformer(paste, analyzer_name)
        if self.attributes:
            # Add extra attributes
            event["Attributes"].extend(self.attributes)
        data = json.dumps({"Event": event})
        # Send event to MISP instance
        r = Request()
        r.headers = {"Authorization": self.access_key, "Accept": "application/json", "Content-Type": "application/json"}
        events_url = "{0}/events".format(self.url)
        res = r.post(events_url, data=data)

        # Error handling
        if not res:
            self.logger.warning("Empty response when adding event")
            return

        res = json.loads(res)
        if "Event" in res:
            event = res.get("Event")
            self.logger.info("Event #%s successfully added to MISP", event.get("id"))
            return
        # An error has happened, but the 'errors' field is not always present
        if "errors" in res:
            self.logger.error("Error when adding event: %s", res.get("errors"))
        self.logger.warning("Failed to add event: %s", res.get("message"))



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.privatekeyanalyzer import PrivateKeyAnalyzer


class TestPrivateKeyAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = PrivateKeyAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        self.paste.body = "-----BEGIN PRIVATE KEY-----asdasdasdfgsdfgdfgs-----END PRIVATE KEY-----"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "-----BEGIN PGP PRIVATE KEY BLOCK-----ZzEF0RDWWYb1vPTIeNrKrMEOZ+LesRil6TXCn5/4I0VinlrnVpRbF4xtsVt4eIRkomN9eN-----END PGP PRIVATE " \
                          "KEY BLOCK-----"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "-----BEGIN RSA PRIVATE KEY-----sRefW7LzPMmTgqHgbNv/gTtT4V/Tc1CiYGVtG9NPlP9tAAAIAI/WvZ3Clh+B0X9mkZ9-----END RSA PRIVATE KEY-----"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "-----BEGIN ENCRYPTED PRIVATE KEY-----sRefW7LzPMmTgqHgbNv/gTtT4V/Tc1CiYGVtG9NPlP9tAAAIAI/WvZ3Clh+B0X9mkZ9-----END ENCRYPTED " \
                          "PRIVATE KEY-----"
        self.assertTrue(self.analyzer.match(self.paste))
        self.paste.body = "-----BEGIN OPENSSH PRIVATE KEY-----sRefW7LzPMmTgqHgbNv/gTtT4V/Tc1CiYGVtG9NPlP9tAAAIAI/WvZ3Clh+B0X9mkZ9-----END OPENSSH " \
                          "PRIVATE KEY-----"
        self.assertTrue(self.analyzer.match(self.paste))

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))
        self.paste.body = "-----BEGIN PUBLIC KEY-----"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer

_EMAIL_PASSWORD_REGEX = r'[\w\.\+_-]+@[\w\._-]+\.[a-zA-Z]*\:[\w\.\+\!\$\#\^&\*\(\)\{\}\[\]\_\-\@\%\=\§\\\/\'\`\´\?\<\>\;\"\:\|\,\~]+'


class EmailPasswordPairAnalyzer(RegexAnalyzer):
    """Analyzer to match username:password pairs"""
    name = "EmailPasswordPairAnalyzer"

    def __init__(self, actions, min_amount=0):
        super().__init__(actions, _EMAIL_PASSWORD_REGEX)
        self.min_amount = min_amount

    def verify(self, results):
        """Method to perform additional checks to test if the matches are actually valid"""
        if len(results) >= self.min_amount:
            return results
        return False



# -*- coding: utf-8 -*-
import logging.handlers

from pastepwn.util import TemplatingEngine
from .basicaction import BasicAction


class SyslogAction(BasicAction):
    """Action to log a paste to the syslog"""
    name = "SyslogAction"

    def __init__(self, syslog_address="/dev/log", template=None):
        """
        This sets up a syslogger, which defaults to /dev/log.
        That means that it will work on most linux systems,
        but will not work on for example OSX.

        For OSX, the correct syslog address would be
        /var/run/syslog
        """
        super().__init__()
        self.template = template
        self.logger = logging.getLogger("SyslogLogger")
        self.logger.setLevel(logging.DEBUG)

        handler = logging.handlers.SysLogHandler(address=syslog_address)
        self.logger.addHandler(handler)

    def perform(self, paste, analyzer_name=None, matches=None):
        """
        Logs a paste to the syslog
        :param paste: The paste passed by the ActionHandler
        :param analyzer_name: The name of the analyzer which matched the paste
        :param matches: List of matches returned by the analyzer
        :return: None
        """
        text = TemplatingEngine.fill_template(paste, analyzer_name, template_string=self.template, matches=matches)
        self.logger.debug(text)



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.analyzers.megalinkanalyzer import MegaLinkAnalyzer


class TestMegaLinkAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = MegaLinkAnalyzer(None)
        self.paste = mock.Mock()

    def test_match_positive(self):
        """Test if positives are recognized"""
        # working mega link (short version)
        self.paste.body = "https://mega.nz/#F!XTQVEAZZ"
        self.assertTrue(self.analyzer.match(self.paste))

        # working mega link (medium version)
        self.paste.body = "https://mega.nz/#F!XTQVEAZZ!eqxlOvTxJKnvAkYvjC0O8g"
        self.assertTrue(self.analyzer.match(self.paste))

        # working mega link (long version)
        self.paste.body = "https://mega.nz/#F!PB8SSawR!SUokSlF2Zy8CR004DNFfNw!LQtniCoa"
        self.assertTrue(self.analyzer.match(self.paste))

        # without https header
        self.paste.body = "mega.nz/#F!XTQVEAZZ!eqxlOvTxJKnvAkYvjC0O8g"
        self.assertTrue(self.analyzer.match(self.paste))

        # http header
        self.paste.body = "http://mega.nz/#F!XTQVEAZZ!eqxlOvTxJKnvAkYvjC0O8g"
        self.assertTrue(self.analyzer.match(self.paste))

        # in a sentence
        self.paste.body = "check out this file:https://mega.nz/#F!PB8SSawR!SUokSlF2Zy8CR004DNFfNw!LQtniCoa"
        self.assertTrue(self.analyzer.match(self.paste))

        # multiple in one paste
        self.paste.body = "check out this file:https://mega.nz/#F!PB8SSawR!SUokSlF2Zy8CR004DNFfNw!LQtniCoa also use this link for other stuff:" \
                          "mega.nz/#F!XTQVEAZZ!eqxlOvTxJKnvAkYvjC0O8g"
        match = self.analyzer.match(self.paste)
        self.assertTrue(match)
        self.assertEqual("https://mega.nz/#F!PB8SSawR!SUokSlF2Zy8CR004DNFfNw!LQtniCoa", match[0])
        self.assertEqual("mega.nz/#F!XTQVEAZZ!eqxlOvTxJKnvAkYvjC0O8g", match[1])

    def test_match_negative(self):
        """Test if negatives are not recognized"""
        self.paste.body = ""
        self.assertFalse(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertFalse(self.analyzer.match(self.paste))

        # Invalid segment length
        self.paste.body = "https://mega.nz/#F!XTQVEAZZ1!eqxlOvTxJKnvAkYvjC0O8g"
        self.assertFalse(self.analyzer.match(self.paste))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import unittest
from unittest import mock

from pastepwn.actions.basicaction import BasicAction
from pastepwn.analyzers.alwaystrueanalyzer import AlwaysTrueAnalyzer


class TestAlwaysTrueAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = AlwaysTrueAnalyzer(None)
        self.paste = mock.Mock()

    def test_match(self):
        self.paste.body = "Test"
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = None
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste.body = ""
        self.assertTrue(self.analyzer.match(self.paste))

        self.paste = None
        self.assertTrue(self.analyzer.match(self.paste))

    def test_actions_present(self):
        action = mock.MagicMock(spec=BasicAction)
        analyzer = AlwaysTrueAnalyzer(action)
        self.assertEqual([action], analyzer.actions)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class GoogleOAuthKeyAnalyzer(RegexAnalyzer):

    def __init__(self, actions):
        regex = r"[0-9]+-[0-9A-Za-z_]{32}\.apps\.googleusercontent\.com"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-

from .andanalyzer import AndAnalyzer
from .logicalbaseanalyzer import LogicalBaseAnalyzer
from .oranalyzer import OrAnalyzer

__all__ = ["LogicalBaseAnalyzer", "AndAnalyzer", "OrAnalyzer"]



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class GoogleApiKeyAnalyzer(RegexAnalyzer):
    """Analyzer that matches Google API Keys via regex"""

    def __init__(self, actions):
        # https://cloud.google.com/docs/authentication/api-keys
        regex = r"\bAIza[0-9A-Za-z_-]{35}\b"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
from .regexanalyzer import RegexAnalyzer


class MailChimpApiKeyAnalyzer(RegexAnalyzer):

    def __init__(self, actions):
        regex = r"[0-9a-f]{32}-us[0-9]{12}"
        super().__init__(actions, regex)



# -*- coding: utf-8 -*-
import logging
import unittest
from unittest import mock

from pastepwn.actions.basicaction import BasicAction
from pastepwn.analyzers.basicanalyzer import BasicAnalyzer
from pastepwn.errors import InvalidActionError


class TestBasicAnalyzer(unittest.TestCase):
    def setUp(self):
        self.analyzer = BasicAnalyzer(None)
        self.mock_action = mock.MagicMock(spec=BasicAction)

    def test_add_action(self):
        """Check if it's possible to add actions to an analyzer"""
        mock_action2 = mock.MagicMock(spec=BasicAction)
        analyzer = BasicAnalyzer(self.mock_action)
        self.assertEqual([self.mock_action], analyzer.actions)

        analyzer.add_action(mock_action2)
        self.assertIsInstance(analyzer.actions, list)
        self.assertEqual([self.mock_action, mock_action2], analyzer.actions)

    def test_add_action_negative(self):
        """Check if an exception raises when passing crap as action"""
        analyzer = BasicAnalyzer(self.mock_action)
        self.assertEqual([self.mock_action], analyzer.actions)

        with self.assertRaises(InvalidActionError, msg="BasicAnalyzer accepts objects other than BasicAnalyzers as actions in add_action!"):
            # The mock_action2 is not a subclass of BasicAction and hence it must throw an error
            mock_action2 = mock.Mock()
            analyzer.add_action(mock_action2)

        with self.assertRaises(InvalidActionError, msg="BasicAnalyzer accepts classes as actions in add_action!"):
            # The mock_action3 is not even an object (only a class reference)
            mock_action3 = mock.Mock
            analyzer.add_action(mock_action3)

    def test_match(self):
        """Check if an exception is raised when trying to call the match function"""
        self.mock_action.body = "Test"
        self.assertRaises(NotImplementedError, self.analyzer.match, self.mock_action)

    def test_initialization(self):
        """Check if the initialization of the BasicAnalyzer works as intended"""
        analyzer = BasicAnalyzer(self.mock_action)
        self.assertEqual([self.mock_action], analyzer.actions)

    def test_empty_initialization(self):
        """Check if initializing an analyzer without actions is possible"""
        analyzer = BasicAnalyzer(None)
        self.assertEqual([], analyzer.actions)

    def test_single_initialization(self):
        """Check if initializing an analyzer with a single action is possible"""
        analyzer = BasicAnalyzer(self.mock_action)
        self.assertEqual([self.mock_action], analyzer.actions)

    def test_multi_initialization(self):
        """Check if initializing an analyzer with multiple actions is possible"""
        mock_action2 = mock.MagicMock(spec=BasicAction)
        actions = [self.mock_action, mock_action2]
        analyzer = BasicAnalyzer(actions)
        self.assertEqual([self.mock_action, mock_action2], analyzer.actions)

    def test_logical_operators(self):
        """Check if using logical operators on analyzers works as intended"""
        analyzer = self.analyzer & self.analyzer
        self.assertIsInstance(analyzer, BasicAnalyzer)

        mock_action1 = mock.MagicMock(spec=BasicAction)
        mock_action2 = mock.MagicMock(spec=BasicAction)
        analyzer1 = BasicAnalyzer(mock_action1)
        analyzer2 = BasicAnalyzer(mock_action2)

        and_analyzer = analyzer1 & analyzer2
        self.assertEqual(2, len(and_analyzer.actions))
        self.assertIn(mock_action1, and_analyzer.actions)
        self.assertIn(mock_action2, and_analyzer.actions)
        self.assertEqual("(BasicAnalyzer && BasicAnalyzer)", and_analyzer.identifier)

    def test_init_action_class(self):
        """Check if an exception is being raised when a passed action is a reference to a class"""
        # Test created based on #175
        # If users pass a class instead of an instance, they should receive an exception
        with self.assertRaises(InvalidActionError, msg="BasicAnalyzer accepts classes as actions!"):
            _ = BasicAnalyzer(mock.Mock)

    def test_error_logging_init_class(self):
        """Check if an error is being logged when a passed action is a reference to a class"""
        self.logger = logging.getLogger("pastepwn.analyzers.basicanalyzer")
        with self.assertLogs(self.logger, level="ERROR") as log:
            try:
                _ = BasicAnalyzer(mock.Mock)
            except Exception:
                pass

            self.assertEqual(log.output, ["ERROR:pastepwn.analyzers.basicanalyzer:You passed a class as action for 'BasicAnalyzer' but an instance of an action was expected!"])

    def test_unique(self):
        """Check if running unique() on a list of matches returns a list with no duplicates."""
        # Some lists with and without duplicates
        test_lists = [
            [],
            ["a", "a"],
            ["a", "b"],
            ["a", "b", "a"]
            ]
        self.assertEqual(BasicAnalyzer.unique(test_lists[0]), [],
                         msg="BasicAnalyzer.unique() left a duplicate!")
        self.assertEqual(BasicAnalyzer.unique(test_lists[1]), ["a"],
                         msg="BasicAnalyzer.unique() left a duplicate!")
        # Should preserve order
        self.assertEqual(BasicAnalyzer.unique(test_lists[2]), ["a", "b"],
                         msg="BasicAnalyzer.unique() left a duplicate!")
        self.assertEqual(BasicAnalyzer.unique(test_lists[3]), ["a", "b"],
                         msg="BasicAnalyzer.unique() left a duplicate!")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
import logging
import re

from pastepwn.util import Request, TemplatingEngine
from .basicaction import BasicAction


class TelegramAction(BasicAction):
    """Action to send a Telegram message to a certain user or group/channel"""
    name = "TelegramAction"

    def __init__(self, token, receiver, template=None):
        """Action to send a Telegram message to a certain user or group/channel.
        :param token: The Telegram API token for your bot obtained by @BotFather
        :param receiver: The userID/groupID or channelID of the receiving entity
        :param template: A template string describing how the paste variables should be filled in
        """
        super().__init__()
        self.logger = logging.getLogger(__name__)

        if not re.match(r"[0-9]+:[a-zA-Z0-9\-_]+", token) or token is None:
            raise ValueError("Bot token not correct or None!")

        self.token = token
        self.receiver = receiver
        self.template = template

    def perform(self, paste, analyzer_name=None, matches=None):
        """Send a message via a Telegram bot to a specified user, without checking for errors"""
        r = Request()
        text = TemplatingEngine.fill_template(paste, analyzer_name, template_string=self.template, matches=matches)

        api_url = "https://api.telegram.org/bot{0}/sendMessage?chat_id={1}&text={2}".format(self.token, self.receiver, text)
        r.get(api_url)
